{% extends "django_tasker_account/base.html" %}
{% block form %}
    {% include "django_tasker_account/form.html" with button="Change" title="Change avatar" enctype="multipart/form-data" %}
{% endblock %}